@@ -32,7 +32,7 @@ Provide the base classes for GRAMPS' DataView classes
 from abc import abstractmethod
 import os
 import pickle
-import time
+from time import perf_counter
 import logging
 from collections import deque
 
@@ -56,7 +56,7 @@ from gramps.gen.const import GRAMPS_LOCALE as glocale
 _ = glocale.translation.sgettext
 from .pageview import PageView
 from .navigationview import NavigationView
-from ..actiongroup import ActionGroup
+from ..uimanager import ActionGroup
 from ..columnorder import ColumnOrder
 from gramps.gen.config import config
 from gramps.gen.errors import WindowActiveError, FilterError, HandleError
@@ -65,7 +65,8 @@ from ..widgets.menuitem import add_menuitem
 from gramps.gen.const import CUSTOM_FILTERS
 from gramps.gen.utils.debug import profile
 from gramps.gen.utils.string import data_recover_msg
-from ..dialog import QuestionDialog, QuestionDialog2, ErrorDialog
+from gramps.gen.plug import CATEGORY_QR_PERSON
+from ..dialog import QuestionDialog, QuestionDialog3, ErrorDialog
 from ..editors import FilterEditor
 from ..ddtargets import DdTargets
 from ..plug.quick import create_quickreport_menu, create_web_connect_menu
@@ -123,6 +124,8 @@ class ListView(NavigationView):
         self.generic_filter = None
         dbstate.connect('database-changed', self.change_db)
         self.connect_signals()
+        self.at_popup_action = None
+        self.at_popup_menu = None
 
     def no_database(self):
         ## TODO GTK3: This is never called!! Dbguielement disconnects
@@ -169,11 +172,6 @@ class ListView(NavigationView):
                                     [self.drag_dest_info().target()],
                                     Gdk.DragAction.MOVE |
                                     Gdk.DragAction.COPY)
-            tglist = Gtk.TargetList.new([])
-            tglist.add(self.drag_dest_info().atom_drag_type,
-                       self.drag_dest_info().target_flags,
-                       self.drag_dest_info().app_id)
-            self.list.drag_dest_set_target_list(tglist)
 
         scrollwindow = Gtk.ScrolledWindow()
         scrollwindow.set_policy(Gtk.PolicyType.AUTOMATIC,
@@ -207,24 +205,19 @@ class ListView(NavigationView):
 
         NavigationView.define_actions(self)
 
-        self.edit_action = ActionGroup(name=self.title + '/ChangeOrder')
+        self.edit_action = ActionGroup(name=self.title + '/Edits')
         self.edit_action.add_actions([
-                ('Add', 'list-add', _("_Add..."), "<PRIMARY>Insert",
-                    self.ADD_MSG, self.add),
-                ('Remove', 'list-remove', _("_Delete"), "<PRIMARY>Delete",
-                    self.DEL_MSG, self.remove),
-                ('Merge', 'gramps-merge', _('_Merge...'), None,
-                    self.MERGE_MSG, self.merge),
-                ('ExportTab', None, _('Export View...'), None, None,
-                    self.export),
-                ])
+            ('Add', self.add, '<Primary>Insert'),
+            ('Remove', self.remove, '<Primary>Delete'),
+            ('PRIMARY-BackSpace', self.remove, '<PRIMARY>BackSpace'),
+            ('Merge', self.merge), ])
 
         self._add_action_group(self.edit_action)
-
-        self._add_action('Edit', 'gtk-edit', _("action|_Edit..."),
-                         accel="<PRIMARY>Return",
-                         tip=self.EDIT_MSG,
-                         callback=self.edit)
+        self.action_list.extend([
+            ('ExportTab', self.export),
+            ('Edit', self.edit, '<Primary>Return'),
+            ('PRIMARY-J', self.jump, '<PRIMARY>J'),
+            ('FilterEdit', self.filter_editor)])
 
     def build_columns(self, preserve_col=True):
         """
@@ -298,7 +291,7 @@ class ListView(NavigationView):
         Called when the page is displayed.
         """
         NavigationView.set_active(self)
-        self.uistate.viewmanager.tags.tag_enable()
+        self.uistate.viewmanager.tags.tag_enable(update_menu=False)
         self.uistate.show_filter_results(self.dbstate,
                                          self.model.displayed(),
                                          self.model.total())
@@ -310,12 +303,9 @@ class ListView(NavigationView):
         NavigationView.set_inactive(self)
         self.uistate.viewmanager.tags.tag_disable()
 
-    def __build_tree(self):
-        profile(self._build_tree)
-
     def build_tree(self, force_sidebar=False, preserve_col=True):
         if self.active:
-            cput0 = time.clock()
+            cput0 = perf_counter()
             if not self.search_bar.is_visible():
                 filter_info = (True, self.generic_filter, False)
             else:
@@ -341,26 +331,26 @@ class ListView(NavigationView):
                     ErrorDialog(msg1, msg2,
                                 parent=self.uistate.window)
 
-            cput1 = time.clock()
+            cput1 = perf_counter()
             self.build_columns(preserve_col)
-            cput2 = time.clock()
+            cput2 = perf_counter()
             self.list.set_model(self.model)
-            cput3 = time.clock()
+            cput3 = perf_counter()
             self.__display_column_sort()
             self.goto_active(None)
 
             self.dirty = False
-            cput4 = time.clock()
+            cput4 = perf_counter()
             self.uistate.show_filter_results(self.dbstate,
                                              self.model.displayed(),
                                              self.model.total())
             LOG.debug(self.__class__.__name__ + ' build_tree ' +
-                    str(time.clock() - cput0) + ' sec')
+                    str(perf_counter() - cput0) + ' sec')
             LOG.debug('parts ' + str(cput1-cput0) + ' , '
                              + str(cput2-cput1) + ' , '
                              + str(cput3-cput2) + ' , '
                              + str(cput4-cput3) + ' , '
-                             + str(time.clock() - cput4))
+                             + str(perf_counter() - cput4))
 
         else:
             self.dirty = True
@@ -379,7 +369,7 @@ class ListView(NavigationView):
         """
         return 'gramps-tree-list'
 
-    def filter_editor(self, obj):
+    def filter_editor(self, *obj):
         try:
             FilterEditor(self.FILTER_TYPE , CUSTOM_FILTERS,
                          self.dbstate, self.uistate)
@@ -448,7 +438,7 @@ class ListView(NavigationView):
             self.uistate.push_message(self.dbstate,
                                       _("Active object not visible"))
 
-    def add_bookmark(self, obj):
+    def add_bookmark(self, *obj):
         mlist = []
         self.selection.selected_foreach(self.blist, mlist)
 
@@ -533,7 +523,7 @@ class ListView(NavigationView):
         """
         order = self._config.get('columns.rank')
         size = self._config.get('columns.size')
-        vis =  self._config.get('columns.visible')
+        vis = self._config.get('columns.visible')
 
         colord = [(1 if val in vis else 0, val, size)
             for val, size in zip(order, size)]
@@ -548,14 +538,18 @@ class ListView(NavigationView):
         """
         prompt = True
         if len(self.selected_handles()) > 1:
-            q = QuestionDialog2(
+            ques = QuestionDialog3(
                 _("Multiple Selection Delete"),
                 _("More than one item has been selected for deletion. "
                   "Select the option indicating how to delete the items:"),
                 _("Delete All"),
                 _("Confirm Each Delete"),
                 parent=self.uistate.window)
-            prompt = not q.run()
+            res = ques.run()
+            if res == -1:  # Cancel
+                return
+            else:
+                prompt = not res  # we prompt on 'Confirm Each Delete'
 
         if not prompt:
             self.uistate.set_busy_cursor(True)
@@ -574,11 +568,16 @@ class ListView(NavigationView):
                 #descr = object.get_description()
                 #if descr == "":
                 descr = object.get_gramps_id()
-                self.uistate.set_busy_cursor(True)
-                QuestionDialog(_('Delete %s?') % descr, msg,
-                               _('_Delete Item'), query.query_response,
-                               parent=self.uistate.window)
-                self.uistate.set_busy_cursor(False)
+                ques = QuestionDialog3(_('Delete %s?') % descr, msg,
+                                       _('_Yes'), _('_No'),
+                                       parent=self.uistate.window)
+                res = ques.run()
+                if res == -1:  # Cancel
+                    return
+                elif res:  # If true, perfom the delete
+                    self.uistate.set_busy_cursor(True)
+                    query.query_response()
+                    self.uistate.set_busy_cursor(False)
             else:
                 query.query_response()
 
@@ -619,7 +618,7 @@ class ListView(NavigationView):
         """
         self.uistate.set_busy_cursor(True)
         self.uistate.push_message(self.dbstate, _("Column clicked, sorting..."))
-        cput = time.clock()
+        cput = perf_counter()
         same_col = False
         if self.sort_col != data:
             order = Gtk.SortType.ASCENDING
@@ -671,7 +670,7 @@ class ListView(NavigationView):
         self.uistate.set_busy_cursor(False)
 
         LOG.debug('   ' + self.__class__.__name__ + ' column_clicked ' +
-                    str(time.clock() - cput) + ' sec')
+                    str(perf_counter() - cput) + ' sec')
 
     def __display_column_sort(self):
         for i, c in enumerate(self.columns):
@@ -725,25 +724,16 @@ class ListView(NavigationView):
         if len(selected_ids) == 1:
             if self.drag_info():
                 self.list.drag_source_set(Gdk.ModifierType.BUTTON1_MASK,
-                                      [],
-                                      Gdk.DragAction.COPY)
-                #TODO GTK3: wourkaround here for bug https://bugzilla.gnome.org/show_bug.cgi?id=680638
-                tglist = Gtk.TargetList.new([])
-                dtype = self.drag_info()
-                tglist.add(dtype.atom_drag_type, dtype.target_flags, dtype.app_id)
-                self.list.drag_source_set_target_list(tglist)
+                                          [self.drag_info().target()],
+                                          Gdk.DragAction.COPY)
         elif len(selected_ids) > 1:
             if self.drag_list_info():
                 self.list.drag_source_set(Gdk.ModifierType.BUTTON1_MASK,
-                                      [],
-                                      Gdk.DragAction.COPY)
-                #TODO GTK3: wourkaround here for bug https://bugzilla.gnome.org/show_bug.cgi?id=680638
-                tglist = Gtk.TargetList.new([])
-                dtype = self.drag_list_info()
-                tglist.add(dtype.atom_drag_type, dtype.target_flags, dtype.app_id)
-                self.list.drag_source_set_target_list(tglist)
+                                          [self.drag_list_info().target()],
+                                          Gdk.DragAction.COPY)
 
-        self.uistate.modify_statusbar(self.dbstate)
+        if self.uistate.viewmanager.active_page == self:
+            self.uistate.modify_statusbar(self.dbstate)
 
     def row_add(self, handle_list):
         """
@@ -751,10 +741,10 @@ class ListView(NavigationView):
         """
         if self.active or \
            (not self.dirty and not self._dirty_on_change_inactive):
-            cput = time.clock()
+            cput = perf_counter()
             list(map(self.model.add_row_by_handle, handle_list))
             LOG.debug('   ' + self.__class__.__name__ + ' row_add ' +
-                    str(time.clock() - cput) + ' sec')
+                    str(perf_counter() - cput) + ' sec')
             if self.active:
                 self.uistate.show_filter_results(self.dbstate,
                                                  self.model.displayed(),
@@ -770,12 +760,12 @@ class ListView(NavigationView):
             self.model.prev_handle = None
         if self.active or \
            (not self.dirty and not self._dirty_on_change_inactive):
-            cput = time.clock()
+            cput = perf_counter()
             #store selected handles
             self._sel_handles_before_update = self.selected_handles()
             list(map(self.model.update_row_by_handle, handle_list))
             LOG.debug('   ' + self.__class__.__name__ + ' row_update ' +
-                    str(time.clock() - cput) + ' sec')
+                    str(perf_counter() - cput) + ' sec')
             # Ensure row is still selected after a change of postion in tree.
             if self._sel_handles_before_update:
                 #we can only set one selected again, we take last
@@ -791,10 +781,20 @@ class ListView(NavigationView):
         """
         if self.active or \
            (not self.dirty and not self._dirty_on_change_inactive):
-            cput = time.clock()
-            list(map(self.model.delete_row_by_handle, handle_list))
-            LOG.debug('   '  + self.__class__.__name__ + ' row_delete ' +
-                    str(time.clock() - cput) + ' sec')
+            cput = perf_counter()
+            for hndl in handle_list:
+                if hndl != handle_list[-1]:
+                    # For multiple deletes, row updates can result in a
+                    # selection changed signal to a handle already
+                    # deleted.  In these cases we don't want to change the
+                    # active to non-existant handles.
+                    self.model.dont_change_active = True
+                else:
+                    # Allow active changed on last item deleted
+                    self.model.dont_change_active = False
+                self.model.delete_row_by_handle(hndl)
+            LOG.debug('   ' + self.__class__.__name__ + ' row_delete ' +
+                      str(perf_counter() - cput) + ' sec')
             if self.active:
                 self.uistate.show_filter_results(self.dbstate,
                                                  self.model.displayed(),
@@ -858,7 +858,9 @@ class ListView(NavigationView):
         """
         if not self.dbstate.is_open():
             return False
-        if event.type == Gdk.EventType._2BUTTON_PRESS and event.button == 1:
+        menu = self.uimanager.get_widget('Popup')
+        if (event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS
+                and event.button == 1):
             if self.model.get_flags() & Gtk.TreeModelFlags.LIST_ONLY:
                 self.edit(obj)
                 return True
@@ -873,49 +875,54 @@ class ListView(NavigationView):
                     else:
                         self.edit(obj)
                         return True
-        elif is_right_click(event):
-            menu = self.uistate.uimanager.get_widget('/Popup')
-            if menu:
-                # Quick Reports
-                qr_menu = self.uistate.uimanager.\
-                            get_widget('/Popup/QuickReport')
-                if qr_menu and self.QR_CATEGORY > -1 :
-                    (ui, qr_actions) = create_quickreport_menu(
-                                            self.QR_CATEGORY,
-                                            self.dbstate,
-                                            self.uistate,
-                                            self.first_selected())
-                    self.__build_menu(qr_menu, qr_actions)
+        elif is_right_click(event) and menu:
+            prefix = 'win'
+            self.at_popup_menu = []
+            actions = []
+            # Quick Reports
+            if self.QR_CATEGORY > -1:
+                (qr_ui, qr_actions) = create_quickreport_menu(
+                    self.QR_CATEGORY, self.dbstate, self.uistate,
+                    self.first_selected(), prefix)
+                if self.get_active() and qr_actions:
+                    actions.extend(qr_actions)
+                    qr_ui = ("<placeholder id='QuickReport'>%s</placeholder>" %
+                             qr_ui)
+                    self.at_popup_menu.append(qr_ui)
 
-                # Web Connects
-                web_menu = self.uistate.uimanager.\
-                                get_widget('/Popup/WebConnect')
-                if web_menu:
-                    web_actions = create_web_connect_menu(
-                                        self.dbstate,
-                                        self.uistate,
-                                        self.navigation_type(),
-                                        self.first_selected())
-                    self.__build_menu(web_menu, web_actions)
+            # Web Connects
+            if self.QR_CATEGORY == CATEGORY_QR_PERSON:
+                (web_ui, web_actions) = create_web_connect_menu(
+                    self.dbstate, self.uistate, self.navigation_type(),
+                    self.first_selected(), prefix)
+                if self.get_active() and web_actions:
+                    actions.extend(web_actions)
+                    self.at_popup_menu.append(web_ui)
 
-                menu.popup(None, None, None, None, event.button, event.time)
-                return True
+            if self.at_popup_action:
+                self.uimanager.remove_ui(self.at_popup_menu)
+                self.uimanager.remove_action_group(self.at_popup_action)
+            self.at_popup_action = ActionGroup('AtPopupActions',
+                                               actions)
+            self.uimanager.insert_action_group(self.at_popup_action)
+            self.at_popup_menu = self.uimanager.add_ui_from_string(
+                self.at_popup_menu)
+            self.uimanager.update_menu()
+
+            menu = self.uimanager.get_widget('Popup')
+            popup_menu = Gtk.Menu.new_from_model(menu)
+            popup_menu.attach_to_widget(obj, None)
+            popup_menu.show_all()
+            if Gtk.MINOR_VERSION < 22:
+                # ToDo The following is reported to work poorly with Wayland
+                popup_menu.popup(None, None, None, None,
+                                 event.button, event.time)
+            else:
+                popup_menu.popup_at_pointer(event)
+            return True
 
         return False
 
-    def __build_menu(self, menu, actions):
-        """
-        Build a submenu for quick reports and web connects
-        """
-        if self.get_active() and len(actions) > 1:
-            sub_menu = Gtk.Menu()
-            for action in actions[1:]:
-                add_menuitem(sub_menu, action[2], None, action[5])
-            menu.set_submenu(sub_menu)
-            menu.show()
-        else:
-            menu.hide()
-
     def _key_press(self, obj, event):
         """
         Called when a key is pressed on a listview
@@ -1009,9 +1016,6 @@ class ListView(NavigationView):
             return True
         return False
 
-    def key_delete(self):
-        self.remove(None)
-
     def change_page(self):
         """
         Called when a page is changed.
@@ -1021,8 +1025,9 @@ class ListView(NavigationView):
             self.uistate.show_filter_results(self.dbstate,
                                              self.model.displayed(),
                                              self.model.total())
-        self.edit_action.set_visible(True)
-        self.edit_action.set_sensitive(not self.dbstate.db.readonly)
+        self.uimanager.set_actions_visible(self.edit_action, True)
+        self.uimanager.set_actions_sensitive(self.edit_action,
+                                             not self.dbstate.db.readonly)
 
     def on_delete(self):
         """
@@ -1052,7 +1057,7 @@ class ListView(NavigationView):
     ####################################################################
     # Export data
     ####################################################################
-    def export(self, obj):
+    def export(self, *obj):
         chooser = Gtk.FileChooserDialog(
             _("Export View as Spreadsheet"),
             self.uistate.window,
@@ -1157,25 +1162,25 @@ class ListView(NavigationView):
     # Template functions
     ####################################################################
     @abstractmethod
-    def edit(self, obj, data=None):
+    def edit(self, *obj):
         """
         Template function to allow the editing of the selected object
         """
 
     @abstractmethod
-    def remove(self, handle, data=None):
+    def remove(self, *obj):
         """
         Template function to allow the removal of an object by its handle
         """
 
     @abstractmethod
-    def add(self, obj, data=None):
+    def add(self, *obj):
         """
         Template function to allow the adding of a new object
         """
 
     @abstractmethod
-    def merge(self, obj, data=None):
+    def merge(self, *obj):
         """
         Template function to allow the merger of two objects.
         """
@@ -1186,7 +1191,7 @@ class ListView(NavigationView):
         Template function to allow the removal of an object by its handle
         """
 
-    def open_all_nodes(self, obj):
+    def open_all_nodes(self, *obj):
         """
         Method for Treeviews to open all groups
         obj: for use of method in event callback
@@ -1199,14 +1204,14 @@ class ListView(NavigationView):
         self.uistate.set_busy_cursor(False)
         self.uistate.modify_statusbar(self.dbstate)
 
-    def close_all_nodes(self, obj):
+    def close_all_nodes(self, *obj):
         """
         Method for Treeviews to close all groups
         obj: for use of method in event callback
         """
         self.list.collapse_all()
 
-    def open_branch(self, obj):
+    def open_branch(self, *obj):
         """
         Expand the selected branches and all children.
         obj: for use of method in event callback
@@ -1221,7 +1226,7 @@ class ListView(NavigationView):
         self.uistate.set_busy_cursor(False)
         self.uistate.modify_statusbar(self.dbstate)
 
-    def close_branch(self, obj):
+    def close_branch(self, *obj):
         """
         Collapse the selected branches.
         :param obj: not used, present only to allow the use of the method in
